/* ----------------- layout ------------------------ */
html,
body {
    padding: 0 !important;
    margin: 0 !important;
    height: 100% !important;
    width: 100% !important;
    overflow: hidden !important;
    text-align: left !important;
    font-size: 14px !important;
    color: #333333;
    transition: background-color 618ms;
}

#app {
    height: 100%;
    width: 100%;
    overflow: auto;
}

hr {
    clear: both;
    border-width: 5px 10px;
    border-style: solid;
    border-color: white #9abddd;
    height: 1px;
    background: #9abddd;
    border-radius: 5px;
}

footer {
    text-align: center;
    font-size: 14px;
    padding-bottom: 20px;
}

footer a {
    color: #086cca;
}

footer a:hover {
    color: red;
}

/* ----------------- common ------------------------ */

.scroll-proxy {
    min-width: 800px;
    max-width: 1200px;
    margin: 0 auto;
}

.select-all {
    user-select: all;
}

.tip {
    color: #88888888;
    font-size: 14px;
}

.anchor {
    opacity: 0.2;
    text-decoration: none;
    font-size: 18px;
    font-weight: bold;
    color: #086cca;
    will-change: opacity;
    transition: opacity 200ms;
}

.anchor:hover {
    color: #ff0000;
}

code {
    color: #006fff;
    font-size: 12px;
    font-family: Consolas, Monospaced, FangSong, serif;
}

.hide {
    display: none;
}

#loading {
    text-align: center;
    font-size: 3em;
    color: #888888;
    display: none;
}

button {
    border: 1px solid #86b2db;
    color: #086cca;
    background: #e8f0f8;
    padding: 3px 5px;
    cursor: pointer;
    font-size: 12px;
    outline: none;
    border-radius: 2px;
    transition: border 200ms, background-color 200ms;
    user-select: none;
}

button:hover {
    box-shadow: 0 0 3px 1px #086cca;
}

button:active {
    background-color: #086cca;
    color: #fff;
}

#ie-tip {
    color: #cc0000;
    background-color: #faf1f1;
    text-align: center;
    padding: 10px 0;
    display: none;
    cursor: help;
}

.unnamed-item {
    color: #aaaaaa;
}

/* ---------------- load error ------------------------*/

.load-error h3 {
    color: #ff7171;
}

.load-error pre,
.load-error p {
    color: #ff0505;
    border-radius: 4px;
    padding: 20px;
    background-color: #ebebeb;
    line-height: 1.5em;
    word-break: break-word;
}

/* ----------------- header ------------------------ */

header {
    padding: 0 10px;
    box-sizing: border-box;
}

header h1 {
    float: left;
}

#app-name {
    display: inline-block;
    font-weight: normal;
    margin-left: 20px;
    width: 400px;
    overflow: hidden;
    text-overflow: ellipsis;
    word-break: keep-all;
    white-space: nowrap;
}

#tools {
    float: right;
    margin-top: 15px;
    display: none;
}

.buttons {
    margin-bottom: 5px;
}

#api-summary {
    font-size: 12px;
    padding: 5px;
    text-align: right;
}

#api-summary .count1 {
    margin-right: 20px;
}

.buttons i {
    display: inline-block;
    vertical-align: middle;
    height: 14px;
    border-left: 1px solid #aaa;
    margin: 0 5px;
}

.options select {
    width: 140px;
    height: 26px;
    margin-right: 20px;
    border-width: 0 0 1px 0;
}

.options {
    font-size: 12px;
}

/* ----------------- api list ------------------------ */

#api-list {
    min-height: 200px;
    padding: 0 10px;
    box-sizing: border-box;
}

details {
    clear: both;
}

details summary {
    cursor: default;
    outline: none;
    font-size: large;
    color: #444;
    display: block;
    padding: 10px;
    background-color: #ffffff;
    transition: background-color 618ms;
}

details summary span:before,
.route-name:before {
    padding: 2px 5px;
    border-radius: 2px;
    margin: 0 5px;
    vertical-align: 1px;
    background: #ececec;
    color: #636363;
}

details summary .module-name:before {
    content: '模块';
    font-size: 13px;
}

.route-name:before {
    content: '路由';
    font-size: 12px;
}

.route-description,
.return-info {
    background-color: #fafafa;
    padding: 10px;
    border-radius: 4px;
    line-height: 1.6;
}

.route-count {
    margin-left: 20px;
    font-size: 12px;
    line-height: 1;
    color: #0977c0;
}

/* ----------------- api item ------------------------ */

#api-list[data-select-type='1']
> ol
> li
> details
> ol
> li
> div:nth-child(2)
> div
> code,
#api-list[data-select-type='2']
> ol
> li
> details
> ol
> li
> div:nth-child(2)
> div
> code
> span:nth-child(2),
#api-list[data-select-type='3']
> ol
> li
> details
> ol
> li
> div:nth-child(2)
> div
> code
> span:nth-child(2)
> span:nth-child(2),
#api-list[data-select-type='4']
> ol
> li
> details
> ol
> li
> div:nth-child(2)
> div
> code
> span:nth-child(2)
> span:nth-child(2)
> span:nth-child(2),
#api-list[data-select-type='5']
> ol
> li
> details
> ol
> li
> div:nth-child(2)
> div
> code
> span:nth-child(2)
> span:nth-child(2)
> span:nth-child(2)
> span:nth-child(2),
#test-panel[data-select-type='1']
> div
> div:nth-child(2)
> div:nth-child(1)
> div
> div
> code,
#test-panel[data-select-type='2']
> div
> div:nth-child(2)
> div:nth-child(1)
> div
> div
> code
> span:nth-child(2),
#test-panel[data-select-type='3']
> div
> div:nth-child(2)
> div:nth-child(1)
> div
> div
> code
> span:nth-child(2)
> span:nth-child(2),
#test-panel[data-select-type='4']
> div
> div:nth-child(2)
> div:nth-child(1)
> div
> div
> code
> span:nth-child(2)
> span:nth-child(2)
> span:nth-child(2),
#test-panel[data-select-type='5']
> div
> div:nth-child(2)
> div:nth-child(1)
> div
> div
> code
> span:nth-child(2)
> span:nth-child(2)
> span:nth-child(2)
> span:nth-child(2) {
    user-select: all;
    border-bottom: 1px dotted;
}

.method {
    text-transform: uppercase;
    color: #e90588;
    padding: 0 5px;
}

.args-table {
    width: 100%;
    border-collapse: collapse;
    font-size: 14px;
}

.args-table caption {
    text-align: left;
    padding-bottom: 5px;
}

.btn-open-test {
    margin-left: 20px;
    font-size: 13px;
    opacity: 0.2;
    will-change: opacity;
    transition: opacity 200ms;
}

.args-table tr:nth-child(2n) {
    background-color: #f9fbfc;
}

.args-table tr:hover td {
    border-bottom-color: rgb(172, 210, 248);
}

.args-table th {
    font-weight: normal;
}

.args-table th,
.args-table td,
.response-header th,
.response-header td,
.header-table th,
.header-table td {
    border: 1px solid #f0f6fc;
    padding: 10px;
}

.arg-name {
    padding: 2px 3px;
}

.arg-type.is-variable {
    color: #d462f2;
}

.required-field {
    color: rgb(211, 5, 5);
}

.api-items li.module-item {
    list-style: simp-chinese-informal;
}

.api-items li.module-item::marker {
    color: #0787fd;
    font-weight: bold;
}

details summary:hover {
    background-color: #eeeeee;
}

details summary:hover .anchor,
details ol li:hover .anchor {
    opacity: 1;
}

details ol li {
    border-left: 1px dotted transparent;
    list-style: decimal-leading-zero;
    padding-left: 5px;
    padding-top: 10px;
}

details ol li::marker {
    color: #0787fd;
}

details ol li:hover {
    border-left-color: #539fdd;
}

details ol li:hover .btn-open-test {
    opacity: 1;
}

details ol li {
    margin-bottom: 40px;
}

details[open] {
    margin-bottom: 20px;
}

details[open] summary {
    position: sticky;
    top: 0;
    border-bottom: 1px solid #eee;
}

.url-info {
    display: grid;
    grid-template-columns: 1fr 80px;
    gap: 10px;
    margin: 15px 0;
}

.addition-info {
    margin: 15px 5px;
}

.info {
    padding: 5px 0;
    border-radius: 2px;
    transition: background-color 200ms;
    will-change: background-color;
}

.info:hover {
    background-color: #fdf4ff;
}

.comment {
    font-size: 12px;
    color: #555555;
}

.comment pre {
    line-height: 1.4;
    margin: 5px 0;
    padding: 0 5px;
}

.route-name {
    font-size: 15px;
    margin-right: 10px;
}

.return-info {
    font-size: 1em;
    line-height: 1.6;
}

.return-info code {
    margin-right: 5px;
}

/* ----------------- test panel ------------------------ */

#test-panel,
#option-panel {
    position: fixed;
    left: 0;
    top: 0;
    right: 0;
    bottom: 0;
    background-color: rgba(0, 0, 0, 0.2);
    align-items: center;
    justify-content: center;
    display: none;
    transition: background-color 618ms;
}

.panel {
    background-color: #ffffff;
    border-radius: 4px;
    transition: width 200ms, height 200ms;
    display: grid;
    grid-template-rows: 45px 1fr;
}

#test-panel .panel {
    min-width: 720px;
    max-width: 1080px;
    width: 80%;
    min-height: 600px;
    max-height: 100%;
}

#test-panel .url-info .method {
    line-height: 28px;
}

#test-panel .url-info {
    grid-template-columns: 60px 1fr 80px;
}

/*#test-panel td {*/
/*    padding: 2px 5px 0 5px;*/
/*}*/

#test-panel .test-row--tool td {
    padding: 5px;
}

.fullscreen.panel {
    max-width: unset !important;
    width: 100% !important;
    height: 100% !important;
}

.panel-heading {
    padding: 10px;
    border-bottom: 1px solid #dddddd;
    user-select: none;
    font-size: 1.1em;
}

.test-lead {
    background-color: #bebdbd;
    color: #fff;
    border-radius: 2px;
    padding: 2px 3px;
    margin-right: 5px;
    font-size: 12px;
}

.panel-heading--buttons {
    float: right;
}

.panel-heading--buttons button {
    border: none;
}

.panel-body {
    padding: 5px 10px;
    overflow: auto;
}

#test-panel input.arg-name,
.arg-value {
    outline: none;
    width: 100%;
    height: 28px;
    padding: 0;
    box-sizing: border-box;
    border-width: 0 0 1px 0;
    border-color: #888888;
    border-style: solid;
    color: rgb(10, 105, 189);
    margin-bottom: 2px;
}

#test-panel input.arg-name:focus,
#test-panel input.arg-name:hover,
#test-panel input.arg-name:active,
.arg-value:focus,
.arg-value:hover,
.arg-value:active {
    border-width: 1px;
    box-shadow: 0 0 3px 2px #bab9b9;
    padding: 5px;
}


#test-panel input[type=file].arg-value:focus,
#test-panel input[type=file].arg-value:hover,
#test-panel input[type=file].arg-value:active {
    padding: 2px;
}

#test-panel select.arg-type {
    height: 26px;
}

textarea.arg-value {
    /* resize: horizontal; */
    /* transition: height 200ms, width 200ms; */
}

textarea.arg-value:focus,
textarea.arg-value:active {
    height: 80px;
    width: 400px;
}

#btn-send-test {
    font-size: 16px;
    letter-spacing: 5px;
}

.response p {
    font-size: 14px;
}

.response-image {
    max-width: 100%;
}

.response-image:hover {
    box-shadow: 0 0 10px 2px #b7b7b7;
    background-color: #ffffff;
}

.response-content {
    background: #ececeb;
    border: none;
    padding: 15px;
    width: 100%;
    box-sizing: border-box;
    outline: none;
    font-size: 12px;
}

.response-content,
.response-content > pre {
    word-break: break-all;
    line-height: 1.5;
    white-space: break-spaces;
}

.required {
    border-color: red;
}

.response-status {
    margin-left: 20px;
}

.status-success {
    color: green;
}

.status-failure {
    color: orange;
}

.response-time,
.response-type,
.response-length {
    margin-left: 20px;
    font-size: 12px;
}

tr.test-row--tool {
    background-color: #fff !important;
}

#btn-add-test-field,
#btn-add-test-header,
.btn-remove-test-field,
.btn-remove-test-header,
.btn-remove-test-header {
    font-size: 12px;
    color: #333;
}

#btn-add-test-field:hover,
#btn-add-test-header:hover {
    color: rgb(17, 136, 237);
}

.btn-remove-test-field:hover,
.btn-remove-test-header:hover {
    color: rgb(235, 90, 90);
}

/* ---------------option panal --------------------------*/

.option-tip {
    padding: 10px;
    color: #555555;
    font-size: 12px;
    text-align: right;
}

.option-tip:before {
    content: '!';
    width: 16px;
    height: 16px;
    line-height: 16px;
    background: #888;
    color: #fff;
    display: inline-block;
    border-radius: 50%;
    text-align: center;
    overflow: hidden;
    font-family: '黑体';
    vertical-align: middle;
}

#option-panel .panel {
    width: 800px;
    height: 480px;
}

#option-panel .options {
    padding: 10px 60px;
}

#option-panel .options > div {
    margin: 10px 0;
}

#option-panel input[type='checkbox'] {
    vertical-align: -2px;
}

#btn-back-to-top {
    position: fixed;
    right: 40px;
    bottom: 20px;
    background-color: rgb(17, 136, 237);
    width: 40px;
    height: 40px;
    border-radius: 4px;
    z-index: 9;
    display: none;
    align-items: center;
    justify-content: center;
    flex-direction: column;
    transition: opacity 200ms, box-shadow 200ms;
    opacity: 0.5;
    cursor: pointer;
    user-select: none;
}

#btn-back-to-top:hover {
    opacity: 0.8;
    box-shadow: 0 0 10px 1px rgba(17, 136, 237, 0.8);
}

#btn-back-to-top:active {
    opacity: 1;
}

#btn-back-to-top:before,
#btn-back-to-top:after {
    content: '';
    display: inline-block;
}

#btn-back-to-top:before {
    width: 15px;
    height: 8px;
    border-style: solid;
    border-color: transparent transparent white transparent;
    border-width: 0 7px 7px 7px;
    box-sizing: border-box;
}

#btn-back-to-top:after {
    width: 3px;
    height: 16px;
    background-color: #fff;
}

/* --------------- theme-dark --------------------*/

.theme-dark {
    background-color: #555555;
    color: #ddd;
}

.theme-dark hr {
    border-color: #555555 #5b666d;
    background-color: #5b666d;
}

.theme-dark details summary {
    background-color: #555555;
    color: #eee;
}

.theme-dark details summary:hover {
    background-color: #626262;
}

.theme-dark details[open] summary {
    border-bottom-color: #4a4a4a;
}

.theme-dark details ol li::marker,
.theme-dark .api-items li.module-item::marker {
    color: #3f86cc;
}

.theme-dark .anchor {
    color: #3f86cc;
}

.theme-dark details summary span:before,
.theme-dark .route-name:before,
.theme-dark .test-lead {
    color: #ececec;
    background-color: #717070;
}

.theme-dark .route-description,
.theme-dark .return-info {
    background-color: #505050;
}

.theme-dark .args-table th,
.theme-dark .args-table td {
    border-color: #606366;
}

.theme-dark .args-table tr:nth-child(2n) {
    background-color: #525252;
}

.theme-dark .args-table tr:hover td {
    border-bottom-color: rgb(123, 137, 143);
}

.theme-dark code {
    color: #b0a529;
}

.theme-dark .method {
    color: #f956b3;
}

.theme-dark .info:hover {
    background-color: #444145;
}

.theme-dark .comment {
    color: #bebebe;
}

.theme-dark .required-field {
    color: #ff7d7d;
}

.theme-dark footer a:hover {
    color: #ff7d7d;
}

.theme-dark .arg-type.is-variable {
    color: #ff57f4;
}

.theme-dark button {
    background-color: #4b4e50;
    border-color: #3e4750;
    color: #489ae7
}

.theme-dark footer a {
    color: #3f86cc;
}

.theme-dark #test-panel,
.theme-dark #option-panel {
    background-color: rgba(29, 29, 29, 0.5);
}

.theme-dark .panel {
    background-color: #555555;
}

.theme-dark .panel .panel-heading {
    border-bottom-color: #474747;
}

.theme-dark input,
.theme-dark textarea,
.theme-dark select {
    background-color: #555555;
    color: #ddd;
    border-color: #474747;
    outline: none;
}

.theme-dark #test-panel table.args-table input:hover,
.theme-dark #test-panel table.args-table input:focus,
.theme-dark #test-panel table.args-table input:active,
.theme-dark #test-panel table.args-table textarea:hover,
.theme-dark #test-panel table.args-table textarea:focus,
.theme-dark #test-panel table.args-table textarea:active {
    box-shadow: 0 0 3px 2px #373535;
}

.theme-dark #test-panel input.arg-name {
    color: #00c7ff;
}

.theme-dark .response-content {
    background-color: #4c4c4c;
}

.theme-dark tr.test-row--tool {
    background-color: #5f5f5f !important;
}

.theme-dark #btn-add-test-field,
.theme-dark .btn-remove-test-field {
    color: #eee;
}

.theme-dark #btn-add-test-field:hover,
.theme-dark .btn-remove-test-field:hover {
    color: #00c7ff;
}

.theme-dark .option-tip {
    color: #c4c3c3;
}

.theme-dark #ie-tip {
    background-color: #360404;
    color: #fb4c4c;
}

.theme-dark .load-error pre,
.theme-dark .load-error p {
    color: #ff5555;
    background-color: #414141;
}

.theme-dark .route-count {
    color: #6bbbf0;
}

.tabs-layout {
    width: 100%;
    margin-bottom: 40px;
}

.tabs-header {
    list-style: none;
    padding: 0;
    border-bottom: 1px solid #cccccc;
    overflow: auto;
    margin: 10px 0 5px;
}

.tabs-header li[data-index] {
    float: left;
    padding: 3px 10px;
    margin-left: 5px;
    cursor: pointer;
    color: #666;
}

.tabs-header li[data-index].active {
    border: 1px solid #cccccc;
    border-bottom: none;
}

.response-meta {
    float: right;
}

.tabs-header li.active,
.tabs-header li:hover {
    color: #333;
}

.tabs-layout .tabs-item {
    width: 100%;
    display: none;
}

.tabs-layout div.tabs-item.active {
    display: block;
}

.tabs-layout table.tabs-item.active {
    display: table;
}

.header-table,
.response-header {
    border-collapse: collapse;
}

.test-header--checkbox {
    cursor: pointer;
}
